#include <QApplication>
#include <QGridLayout>
#include <QPushButton>
#include <QPointer>
#include <QWindow>
#include <QScreen>
#include <QWidget>
#include <QDebug>

#include <QtX11Extras/QX11Info>
#include <xcb/xcb_icccm.h>

class W : public QWidget
{
public:
    W()
    {
        create();
        auto win = windowHandle();
        Q_ASSERT(win);

        // Create a simple layout for visual testing
        auto l = new QGridLayout(this);
        for (int i = 0; i < 10; ++i)
            l->addWidget(new QPushButton(QString("Text %1").arg(i)));

        setMaximumSize(300, 800);

        connect(win, &QWindow::screenChanged,
                this, &W::logScreen,
                Qt::QueuedConnection);
    }
    ~W()
    {}

    void logScreen(const QPointer<QScreen> &screen)
    {
        if (!screen)
            return;

        QString x11SizeHints;

        if (auto conn = QX11Info::connection())
        {
            xcb_size_hints_t sizeHints = {};
            auto cookie = xcb_icccm_get_wm_normal_hints(conn, winId());
            if (xcb_icccm_get_wm_normal_hints_reply(conn, cookie, &sizeHints, nullptr) == 1)
            {
                QDebug d(&x11SizeHints);
                d.nospace();
                d << ", X11 minimum size hint: "
                  << QSize(sizeHints.min_width, sizeHints.min_height)
                  << ", X11 maximum size hint: "
                  << QSize(sizeHints.max_width, sizeHints.max_height);
            }
        }

        qDebug().noquote().nospace()
                << screen << " " << screen->geometry()
                << " DPR: " << screen->devicePixelRatio()
                << x11SizeHints;
    }

    void showEvent(QShowEvent *e) override
    {
        QWidget::showEvent(e);
        if (!m_shown)
        {
            logScreen(windowHandle()->screen());
            m_shown = true;
        }
    }

    bool m_shown = false;
};

int main(int argc, char *argv[])
{
    qInstallMessageHandler([](QtMsgType t, const QMessageLogContext &c, const QString &m) {
        fprintf(stderr, "%s\n", qUtf8Printable(qFormatLogMessage(t, c, m)));
        fflush(stderr);
    });

    qDebug().nospace() << "Compiled with: Qt " << QT_VERSION_STR << ", run with: Qt " << qVersion();

    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
    QApplication app(argc, argv);

    W w;
    w.show();

    return app.exec();
}
